import abc
from collections.abc import Generator
from typing import Any

from package_schemes import Package
from pydantic import BaseModel


class PackageVulnerability(BaseModel):
    advisory: str
    cve: str
    id: str
    more_info_path: str
    specs: list[str]
    v: str


class BaseScanProvider(abc.ABC):
    @abc.abstractmethod
    def get_package_vulnerability(self, package: Package) -> Generator[PackageVulnerability, Any, Any]:
        pass



import importlib.util
from collections.abc import Generator
from pathlib import Path
from typing import Any

import ijson
from package_schemes import Package
from packaging.specifiers import SpecifierSet
from packaging.version import Version

from uv_audit.scan.base import BaseScanProvider, PackageVulnerability


class SafetyScanProvider(BaseScanProvider):
    def __init__(self) -> None:
        safety_db_dir = Path(importlib.util.find_spec('safety_db').origin).parent  # type: ignore

        self.insecure_path = safety_db_dir / 'insecure.json'
        self.insecure_full_path = safety_db_dir / 'insecure_full.json'

    def __get_record_in_insecure(self, key: str) -> list[str] | None:
        with open(self.insecure_path, encoding='utf-8') as file:
            for package_name, value in ijson.kvitems(file, ''):
                if package_name == key:
                    return list(value)
        return None

    def __get_record_in_insecure_full(self, key: str) -> list[dict[str, Any]] | None:
        with open(self.insecure_full_path, encoding='utf-8') as file:
            for package_name, value in ijson.kvitems(file, ''):
                if package_name == key:
                    return value
        return None

    def get_package_vulnerability(self, package: Package) -> Generator[PackageVulnerability, Any, Any]:
        # def get_package_vulnerability(self, package: Package) -> Generator[PackageVulnerability, Any, Any]:
        specifier_versions = self.__get_record_in_insecure(package.name)

        if specifier_versions is None:
            return

        target_version = Version(package.version)

        founded = False
        for specifier_version in specifier_versions:
            specifier = SpecifierSet(specifier_version)

            if target_version in specifier:
                founded = True
                break

        if not founded:
            return

        v1 = self.__get_record_in_insecure_full(package.name)

        if v1 is None:
            return

        for vulnerability_spec in v1:
            for specifier_version in vulnerability_spec['specs']:
                specifier = SpecifierSet(specifier_version)

                if target_version in specifier:
                    yield PackageVulnerability.model_validate(vulnerability_spec)



from pathlib import Path

from package_schemes.interfaces import Project, UvLockV1
from package_schemes.interfaces.pyproject import Pyproject
from rich.console import Console, Group
from rich.markdown import Markdown
from rich.padding import Padding
from rich.panel import Panel
from rich.tree import Tree

from uv_audit.scan.safery import SafetyScanProvider

console = Console()

tree = Tree('Rich Tree')


class UvAuditManager(object):
    def __init__(self) -> None:
        path = Path.cwd()
        self.scan_manager = SafetyScanProvider()
        self.project = Project(
            Pyproject(path / 'pyproject.toml'),
            UvLockV1(path / 'uv.lock'),
        )

    def set_options(self, verbose: bool, ignore_codes: list[str], scan_extra_deps: bool, scan_dev_deps: bool):
        pass

    def scan(self):
        total_error = 0
        for package in self.project.get_packages():
            for vulnerability in self.scan_manager.get_package_vulnerability(package):
                total_error += 1

                url = f'https://pyup.io{vulnerability.more_info_path}'
                da = Group(
                    f'[bright_cyan italic]{url}',
                    Padding(Markdown(markup=f'> {vulnerability.advisory}'), (0, 0, 1, 0)),
                    'Affected versions: [bright_yellow]'
                    + '[/bright_yellow] | [bright_yellow]'.join(vulnerability.specs),
                )
                console.print(
                    Panel(
                        da,
                        title_align='left',
                        title=f'[bold bright_red]{package.name}[/bold bright_red] '
                        f'[bright_yellow]{package.version}[/bright_yellow] - '
                        f'[bold bright_red]{vulnerability.cve}[/bold bright_red]',
                    )
                )
        return total_error



from pprint import pp

import pytest
from typer.testing import CliRunner

from uv_audit.cli import cli


class TestCli(object):
    @pytest.fixture()
    def runner(self):
        return CliRunner()

    def test_root(self, runner: CliRunner):
        result = runner.invoke(cli)
        assert result.exit_code == 0

    def test_version(self, runner: CliRunner):
        result = runner.invoke(cli, ['--version'])
        pp(result.output)
        assert result.exit_code == 0



import typer

import uv_audit
from uv_audit.manager import UvAuditManager

cli = typer.Typer(help='uv-audit CLI')


uv_audit_manager = UvAuditManager()


def version_callback(value: bool):
    if value:
        print(f'Version of uv-audit is {uv_audit.__version__}')
        raise typer.Exit(0)


@cli.command()
def command(  # noqa: C901
    ctx: typer.Context,
    #
    version: bool = typer.Option(
        False,
        '--version',
        callback=version_callback,
        help='Print version of uv-audit.',
        is_eager=True,
    ),
    #
    verbose: bool = typer.Option(
        False,
        '--verbose',
        '-v',
        help='Run with verbose',
    ),
    #
    ignore: list[str] | None = typer.Option(
        None,
        '--ignore',
        help='Ignore codes',
    ),
    # Setters
    extra: bool = typer.Option(
        True,
        '--extra',
        help='TODO',
        metavar='by default',
        rich_help_panel='uv-audit Options',
        show_default=False,
    ),
    dev: bool = typer.Option(
        True,
        '--dev',
        help='TODO',
        metavar='by default',
        rich_help_panel='uv-audit Options',
        show_default=False,
    ),
):
    ignore_codes = []

    if ignore is not None:
        ignore_codes = [item for i in ignore for item in i.split(',')]

    uv_audit_manager.set_options(
        verbose=verbose,
        ignore_codes=ignore_codes,
        scan_extra_deps=dev,
        scan_dev_deps=dev,
    )

    errors = uv_audit_manager.scan()

    if errors > 1:
        raise typer.Exit(1)

    raise typer.Exit(0)
